/*
  interpolation.hh
  Author: Luca Rottoli, 2016
  modified by Marco Bonvini, 2017
*/

#pragma once

#include <cmath>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <map>

#include <gsl/gsl_spline.h>
#include <gsl/gsl_sf.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_complex_math.h>

#include "math/matrix.hh"

//integration routines
#include "integration.hh"
#include <cuba.h>

using namespace std;

#include <complex>

using namespace std;

#include "math/complex_functions.hh"
#include "math/hyp_2F1.hh"

namespace HELL {

class Interpolator {
public:
  Interpolator(double xim, unsigned int npoints = 500);  // xim=(m/Q)^2
  ~Interpolator();
  Interpolator(Interpolator& other );
  //
  inline void   Setxim(double xim){ fxim = xim; }
  inline double Getxim()          { return fxim; }
  inline void   GetxiLimits(double &min, double &max) { min=ximin; max=ximax; return; }
  //
  double C2  (double xi) { return C("2"  , xi); }
  double C2ax(double xi) { return C("2ax", xi); }
  double CL  (double xi) { return C("L"  , xi); }
  double CLax(double xi) { return C("Lax", xi); }
  //
  double C2CC(double xi) { return C("2cc", xi); }
  double CLCC(double xi) { return C("Lcc", xi); }
  double C3CC(double xi) { return C("3cc", xi); }
  //
  double InverseMellinC(string type, double xi);
  double Csmallxi(string type, double xi);
  double Clargexi(string type, double xi);
  //double sing(string type, double xi);
  //
private:
  //
  //Interpolator& operator=( const Interpolator& other );
  int fNPoints;
  double fxim, ximin, ximax;
  map<string,gsl_spline*> fSplineCa;
  //map<string,gsl_spline*> fSplineCb;
  double f_reM0, f_slope; // inverse Mellin parameters
  //
protected:
  //  
  double xi_sample(int i);
  //
  friend double integrandC(double t, void *pars);
  //dcomplex singMellin(string type, dcomplex M);
  dcomplex mellinC(string type, dcomplex M);
  double C(string type, double xi);
  //
  // old stuff
  //inline double C2AnalyticV1(double xi){return C2CUBA(xi);};
  //
  //inline double C20(double xi){return C2masslessCUBA(xi);};
  //inline double CL0(double xi){return CLmasslessCUBA(xi);};
  //
  //friend int integrandC2CUBA(const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
  //double C2CUBA(double);
  //double C2V1(double, double, double);
  //
  //friend int integrandC2masslessCUBA(const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
  //double C2masslessCUBA(double xi);
  //double C20(double x1,double x2, double xi);
  //
  //friend int integrandCLmasslessCUBA(const int *ndim, const double x[], const int *ncomp, double res[], void *pars);
  //double CLmasslessCUBA(double xi);
  //double CL0(double x1,double x2, double xi);
  //
};
};
